//
//  GameViewController.m
//  Macoun2014SolarSystemDemo
//
//  Created by Christoph Wick on 24/09/14.
//  Copyright (c) 2014 Christoph Wick, i4innovation GmbH. All rights reserved.
//

#import "GameViewController.h"

@implementation GameViewController

-(void)awakeFromNib
{
    // create a new scene
    SCNScene *scene = [SCNScene scene];

    // create and add a camera to the scene
    SCNNode *cameraNode = [SCNNode node];
    cameraNode.camera = [SCNCamera camera];
	cameraNode.camera.xFov = 45.0;
	cameraNode.position = SCNVector3Make(0, 2, 15);
	cameraNode.rotation = SCNVector4Make(1.0, 0.0, 0.0, -M_PI/20);
    [scene.rootNode addChildNode:cameraNode];
    
	
    // create and add the light that comes out of the sun in every direction
    SCNNode *sunLightNode = [SCNNode node];
    sunLightNode.light = [SCNLight light];
    sunLightNode.light.type = SCNLightTypeOmni;
	sunLightNode.light.color = [NSColor colorWithCalibratedRed:1.0 green:1.0 blue:1.0 alpha:0.0];
    sunLightNode.position = SCNVector3Make(0, 0, 0);
	[scene.rootNode addChildNode:sunLightNode];

	
	// create and add an additional light that lights the sun
	SCNNode *sunLightNode2 = [SCNNode node];
	sunLightNode2.light = [SCNLight light];
	sunLightNode2.light.type = SCNLightTypeSpot;
	sunLightNode2.light.color = [NSColor colorWithCalibratedRed:1.0 green:1.0 blue:1.0 alpha:0.0];
	sunLightNode2.position = SCNVector3Make(0, 10, 10);
	sunLightNode2.rotation = SCNVector4Make(1.0, 0.0, 0.0, -M_PI_4);
	[scene.rootNode addChildNode:sunLightNode2];

	
    // create and add an gray ambient light that light everything not illuminated by the sunlight
    SCNNode *ambientLightNode = [SCNNode node];
    ambientLightNode.light = [SCNLight light];
    ambientLightNode.light.type = SCNLightTypeAmbient;
    ambientLightNode.light.color = [NSColor grayColor];
    [scene.rootNode addChildNode:ambientLightNode];

	
	// Create the sun geometry and node
	SCNSphere *sun = [SCNSphere sphereWithRadius:1.0];
	sun.firstMaterial.diffuse.contents = [NSColor yellowColor];
	
	SCNNode *sunNode = [SCNNode nodeWithGeometry:sun];
	sunNode.position = SCNVector3Make(0.0, 0.0, 0.0);
	[scene.rootNode addChildNode:sunNode];
	
	
	// Create the earth geometry and nodes
	SCNNode *earthAroundSunRotationNode = [SCNNode node];
	earthAroundSunRotationNode.position = SCNVector3Make(0.0, 0.0, 0.0);
	[scene.rootNode addChildNode:earthAroundSunRotationNode];

	SCNNode *earthFromSunTranslationNode = [SCNNode node];
	earthFromSunTranslationNode.position = SCNVector3Make(5.0, 0.0, 0.0);
	[earthAroundSunRotationNode addChildNode:earthFromSunTranslationNode];
	
	SCNNode *earthAroundItselfRotationNode = [SCNNode node];
	[earthFromSunTranslationNode addChildNode:earthAroundItselfRotationNode];
	
	SCNSphere *earth = [SCNSphere sphereWithRadius:0.5];
	earth.firstMaterial.diffuse.contents = [NSImage imageNamed:@"earthmap1k.jpg"];
	earthAroundItselfRotationNode.geometry = earth;

	// Create the moon geometry and nodes
	SCNNode *moonAroundEarthRotationNode = [SCNNode node];
	[earthFromSunTranslationNode addChildNode:moonAroundEarthRotationNode];

	SCNNode *moomFromEarthTranslatioNode = [SCNNode node];
	moomFromEarthTranslatioNode.position = SCNVector3Make(1.0, 0.0, 0.0);
	[moonAroundEarthRotationNode addChildNode:moomFromEarthTranslatioNode];
	
	SCNNode *moonNode = [SCNNode node];
	SCNSphere *moonGeometry = [SCNSphere sphereWithRadius:0.2];
	moonGeometry.firstMaterial.diffuse.contents = [NSImage imageNamed:@"moonmap1k.jpg"];
	
	moonNode.geometry = moonGeometry;
	
	[moomFromEarthTranslatioNode addChildNode:moonNode];
	
	
    // animate earth around sun
    CABasicAnimation *yearAnimation = [CABasicAnimation animationWithKeyPath:@"rotation"];
    yearAnimation.toValue = [NSValue valueWithSCNVector4:SCNVector4Make(0, -1, 0, M_PI*2)];
    yearAnimation.duration = 10;
    yearAnimation.repeatCount = MAXFLOAT; //repeat forever
    [earthAroundSunRotationNode addAnimation:yearAnimation forKey:nil];

	// animate the earth around itself
	CABasicAnimation *dayAnimation = [CABasicAnimation animationWithKeyPath:@"rotation"];
	dayAnimation.toValue = [NSValue valueWithSCNVector4:SCNVector4Make(0, 1, 0, M_PI*2)];
	dayAnimation.duration = 1;
	dayAnimation.repeatCount = MAXFLOAT; //repeat forever
	[earthAroundItselfRotationNode addAnimation:dayAnimation forKey:nil];

	// animate the moon around the earth
	CABasicAnimation *monthAnimation = [CABasicAnimation animationWithKeyPath:@"rotation"];
	monthAnimation.toValue = [NSValue valueWithSCNVector4:SCNVector4Make(0, 1, 0, M_PI*2)];
	monthAnimation.duration = 3;
	monthAnimation.repeatCount = MAXFLOAT; //repeat forever
	[moonAroundEarthRotationNode addAnimation:monthAnimation forKey:nil];
	
	
    // set the scene to the view
    self.gameView.scene = scene;
    
    // allows the user to manipulate the camera
    self.gameView.allowsCameraControl = YES;

    // show statistics such as fps and timing information
    self.gameView.showsStatistics = YES;
    
    // configure the view
    self.gameView.backgroundColor = [NSColor blackColor];  
}

@end
